;BLOCK!
;Allows user to list blocks in current drawing using wildcards, "?" and "*",
;in similar format as used in DOS commands.
;
;
;by
;Wing Tong
;Designers' Aid
;666-8th Avenue, #1
;San Francisco, California 94118
;           Completed:  08-26-88
;            Revision:  N/A
;
(defun C:BLOCK! (/ strlist1 strlist2 str1 str2   ;SET LOCAL VARIABLES
                   number blkname1 blkname2      ;
                   blklist1 lngth1 wild1 wild2   ;
                   char1 atom1 total)            ;
  (setq number 1                                 ;
        total 0                                  ;
  )                                              ;
  (setq str1 (strcase                            ;PROMPT USER FOR BLOCK
               (getstring "\nBlock to list: ")   ;NAME TO LIST USING
             )                                   ;WILD CARDS
  )                                              ;
  (textscr)                                      ;
  (repeat (strlen str1)                          ;BREAK USER-SUPPLIED BLOCK
    (if (= strlist1 nil)                         ;NAME INTO CHARACTERS AND
      (setq strlist1                             ;STORE AS A LIST IN VARIABLE
        (list (substr str1 number 1))            ;STRLIST1 TO BE MANIPULATED
      )                                          ;LATER
      (setq strlist1                             ;
        (cons (substr str1 number 1) strlist1)   ;
      )                                          ;
    )                                            ;
    (setq number (1+ number))                    ;
  )                                              ;
  (setq number 0                                 ;
        strlist1 (reverse strlist1)              ;REVERSE STRLIST1 BECAUSE
  )                                              ;"CONS" HAD CREATED A
                                                 ;REVERSED LIST
                                                 ;
  (setq char1 (nth number strlist1))             ;SET CHAR1 TO FIRST LETTER
                                                 ;IN LIST TO BE USED FOR
                                                 ;MANIPULATION
                                                 ;
  (while (and (/= (length strlist1) number)      ;GO THROUGH EACH LETTER IN
              (/= "*" char1)                     ;LIST UNTIL "*" IS ENCOUNTERED
         )                                       ;ONCE "*" IS ENCOUNTERED, ANY
                                                 ;SUBSEQUENT CHARACTER IS
                                                 ;NEGLIGIBLE AND DISCARDED
                                                 ;
    (if (/= char1 "?")                           ;REWRITE USER-SUPPLIED BLOCK
      (progn                                     ;NAME SUCH THAT IF CHAR1 IS
        (if (= str2 nil)                         ;NOT EQUAL TO "?" USE CHAR1
          (setq str2 char1)                      ;AS LETTER IN STRING
          (setq str2 (strcat str2 char1))        ;VARIABLE STR2, EFFECTIVELY
        )                                        ;PURGING "?" FROM INITIAL
      )                                          ;USER-SUPPLIED BLOCK NAME
                                                 ;
      (progn                                     ;ALSO, MARK THE SPOT WHERE
        (if (= wild1 nil)                        ;"?" IS FOUND IN LIST SO THAT
          (setq wild1 (list (1+ number)))        ;WE KNOW WHICH POSITION IN
          (setq wild1 (cons (1+ number) wild1))  ;USER-SUPPLIED BLOCK NAME USES
        )                                        ;"?" AND LATER CHECK
      )                                          ;FOR THAT SPOT WHEN
    )                                            ;MANIPULATING BLOCK NAMES
                                                 ;USED IN CURRENT DRAWING.
    (setq number (1+ number))                    ;
    (setq char1 (nth number strlist1))           ;
  )                                              ;
  (if (= char1 "*")                              ;IF "*" IS ENCOUNTERED, MARK
    (setq wild2 (1+ number))                     ;POSITION IN USER-SUPPLIED
  )                                              ;BLOCK NAME SO THAT LATER,
                                                 ;WE KNOW HOW MANY CHARACTER
                                                 ;SPACES TO MANIPULATE IN BLOCK
                                                 ;NAMES USED IN CURRENT DRAWING
                                                 ;
  (setq atom1 (tblnext "BLOCK" 1))               ;USE FIRST BLOCK IN CURRENT
                                                 ;DRAWING FOR MANIPULATION
  (while atom1                                   ;
    (setq blkname1 (cdr (assoc 2 atom1)))        ;GET NAME OF BLOCK IN CURRENT
    (setq lngth1 (1+ (strlen blkname1)))         ;DRAWING FOR MANIPULATION
    (setq number 1)                              ;
    (setq strlist2 nil)                          ;
    (while (and (/= lngth1 number)               ;MANIPULATE EACH BLOCK NAME
                (/= number wild2)                ;IN CURRENT DRAWING UNTIL
            )                                    ;EITHER ALL CHARACTERS HAVE
                                                 ;BEEN CONSIDERED, OR WILD2
                                                 ;OR "*" IS ENCOUNTERED
                                                 ;
      (if (not (member number wild1))            ;CREATE LIST SO THAT
        (if (= strlist2 nil)                     ;CHARACTER IN POSITION(S)
          (setq strlist2                         ;MATCHING WILD1 OR "?" IS
            (list (substr blkname1 number 1))    ;PURGED
          )                                      ;
          (setq strlist2                         ;
            (cons                                ;
              (substr blkname1 number 1) strlist2;
            )                                    ;
          )                                      ;
        )                                        ;
      )                                          ;
      (setq number (1+ number))                  ;
    )                                            ;
    (setq blkname2 nil                           ;
          number 0                               ;
          strlist2 (reverse strlist2)            ;REVERSE STRLIST1 BECAUSE
    )                                            ;"CONS" HAD CREATED A
                                                 ;REVERSED LIST
                                                 ;
    (repeat (length strlist2)                    ;REWRITE BLOCK NAME WITH
      (if (= blkname2 nil)                       ;NEW LIST OF CHARACTERS
        (setq blkname2 (nth number strlist2))    ;HAVING PURGED CHARACTERS IN
        (setq blkname2                           ;POSITIONS MATCHING WILD1 OR
          (strcat blkname2 (nth number strlist2));"?"
        )                                        ;
      )                                          ;
      (setq number (1+ number))                  ;
    )                                            ;
                                                 ;
    (if (= blkname2 str2)                        ;COMPARE REWRITTEN BLOCK
      (progn                                     ;NAME IN CURRENT DRAWING TO
        (princ (strcat blkname1 "\n"))           ;THAT OF REWRITTEN USER-
        (setq total (1+ total))                  ;SUPPLIED BLOCK NAME AND IF
      )                                          ;NAMES MATCH, PRINT NAME ON
    )                                            ;SCREEN AND ADD TO TOTAL
                                                 ;NUMBER OF MATCHES
                                                 ;NOTE THAT THIS IS WHERE
                                                 ;YOU CAN PERFORM WHATEVER
                                                 ;ACTION YOU WANT WITH THOSE
                                                 ;MATCHING BLOCK NAMES
                                                 ;
    (setq atom1 (tblnext "BLOCK"))               ;GET NEXT BLOCK NAME TO
                                                 ;MANIPULATE
  )                                              ;
  (if (/= total 0)                               ;IF MATCHING BLOCK NAME
    (princ (strcat "\n" (rtos total 2 0) " user ";EXIST, PRINT NUMBER OF
             (if (> total 1) "blocks" "block")   ;MATCHING BLOCK(S)
           )                                     ;
    )                                            ;
    (princ "\nBlock name not found.")            ;ELSE, PRINT THIS MESSAGE
  )                                              ;
  (princ)                                        ;END WITHOUT NIL
)                                                ;