;
; -------1---------2---------3---------4---------5---------6---------7
;
;   Listing 1.  GHANTT Chart program
;               by Greg MacGeorge and Bill Kramer
;   Advanced AutoLISP Concepts.
;
(defun C:GHANTT ( / Months Prmx Endmx Incr MtextY WtextY LineY LineY2
                    Strtmo Month Stlist)
   (prompt "\nGHANTT Chart setup")
   (ghant_set_up)
   (setq Months (list
                  (list "January" 31)
                  (list "Febuary" 28)
                  (list "March"   31)
                  (list "April"   30)
                  (list "May"     31)
                  (list "June"    30)
                  (list "July"    31)
                  (list "August"  31)
                  (list "September" 30)
                  (list "October" 31)
                  (list "November" 30)
                  (list "December" 31))
         Incr (/ 24.375 365)
         Strtmo (1+ (getklist "\nMonth to start with:"
                              (mapcar 'car Months)
                              (caar Months))))
   (prompt "\nBuilding month column headings...")
   (repeat (1- Strtmo)
      (setq Months (append (cdr Months) (list (car Months)))))
   (ghant_tasks (month_heads Months Incr))
)
; -------1---------2---------3---------4---------5---------6---------7
;
;   Listing 2  GHANTT functions.
;
(defun ghant_set_up ()
   (setvar "CMDECHO" 0)
   (setvar "LIMMIN" (list 0 0))
   (setvar "LIMMAX" (list 36 24))
   (command "ZOOM" "A" "LINE"
    (list 0.625 0.625) (list 0.625 21.375) (list 33.125 21.375)
    (list 33.125 0.625) "C"
    "LINE"
    (list 8.625 21.375) (list 8.625 19.5) (list 0.625 19.5) ""
    "LINE"
    (list 0.625 19.375) (list 8.625 19.375) (list 8.625 0.625) ""
    "LINE"
    (list 8.75 21.375) (list 8.75 19.5) (list 33.125 19.5) ""
    "LINE"
    (list 33.125 19.375) (list 8.75 19.375) (list 8.75 0.625) ""
    "LINE"
    (list 8.75 20.5) (list 33.125 20.5) "")
)
;
(defun month_heads (Months Incr / STlist Endmx Prmx)
   (setq Endmx 0.0 Prmx 0.0)
   (foreach Month Months
       (setq Endmx (+ Endmx (* Incr (cadr Month)))
             Pt1 (list (+ 8.75 Endmx) 19.5))
       (command "COLOR" "WHITE"
                "LINE" Pt1
                (polar Pt1 (/ PI 2) 1.0) ""
                "COLOR" "YELLOW"
                "TEXT" "C"
                (list (+ 8.75 Prmx (/ (- Endmx Prmx) 2.0)) 20.1875)
                0.1875 0.0 (car month))
       (setq Stlist (cons Prmx Stlist)
             Prmx Endmx)
   )
   (command "COLOR" "WHITE")
   (reverse Stlist)
)
;
(defun ghant_tasks (Stlist / CLR TY LY DOIT SMON SDAY ENDMX TASK PT1 PT2)
   (initget 0 "F O")
   (cond ((= (getkword "\nType of bars: (F)illed/(O)pen <F>:") "O")
            (setvar "FILLMODE" 0))
         (t (setvar "FILLMODE" 1)))
   (setq EndMX 0
         Smon 0
         Sday 1
         TY 19.0
         CLR "White"
         LY 19.125
         Doit 1)

   (prompt "\nEnter Task descriptive names, blank to exit.")
   (while Doit
      (setq Task (substr (getstring 1 "\nTask name <40 char max>:") 1 40))
      (cond
        ((= Task "") (setq Doit nil))
        (t
            (initget 0 "White Yellow Blue Green Red Cyan Magenta")
            (setq TMP (getkword (strcat
                "\nColor of entry <" CLR ">: ")))
            (if TMP (setq CLR TMP))
            (command "COLOR" CLR "TEXT" "R" (list 8.5 TY) 0.25 0 TASK)
            (setq
                  Smon (getklist "\nTask starting month"
                                 (mapcar 'car Months)
                                 (car (nth Smon Months)))
                  Endmx (nth Smon STlist)
                  Sday (getint_range "\nTask starting day"
                                     1 (cadr (nth Smon Months)) Sday)
                  Pt1 (list (+ 8.75 (* Sday Incr) Endmx) LY)
                  Smon (getklist "\nTask ending month"
                                 (mapcar 'car Months)
                                 (car (nth Smon Months)))
                  Endmx (nth Smon STlist)
                  Sday (getint_range "\nTask ending day"
                                     1 (cadr (nth Smon Months)) Sday)
                  Pt2 (list (+ 8.75 (* Sday Incr) Endmx) LY))
            (command "TRACE" 0.25 Pt1 Pt2 "")
            (setq TY (- TY 0.375)
                  LY (- LY 0.375))
         ))
   )
)
; -------1---------2---------3---------4---------5---------6---------7
;
;   Listing 3  From the programming library
;
(defun getint_range (prmpt mnv mxv dflt / tmp)
   (setq Tmp (1- Mnv))
   (while (not (<= Mnv Tmp Mxv))
      (setq Tmp (getint (strcat prmpt " ["
                                (itoa Mnv) "-" (itoa Mxv) "] <"
                                (itoa Dflt) ">: ")))
      (if (null Tmp) (setq Tmp Dflt)))
   (setq Tmp Tmp)
)
;
(defun getklist (prmpt Klist Dflt / Tmp Key)
   (setq Tmp "")
   (foreach Key Klist
      (setq Tmp (strcat Tmp " " Key)))
   (initget 0 (substr Tmp 2))
   (setq Tmp (getkword (strcat prmpt " <" Dflt ">: ")))
   (if (null Tmp) (setq Tmp Dflt))
   (- (length Klist)
      (length (member Tmp Klist)))
)
