;--------1---------2---------3---------4---------5---------6
;
;                         LISTING 1
;
;   GROUT:   Graphic Export           AutoLISP Concepts
;   Author:  Bill Kramer
;
;   Purpose:  Output the graphic exchange file for processing
;   by GRIMP in another version of AutoCAD or to another 
;   program.  GROUT explodes all complex entities to their
;   elementary level [ie: Lines and Arcs]. 
;
(defun r2d (r)
  (* (/ r pi) 180.0))
;
(defun C:GROUT ()
   (setq EN (entnext)
         FN (getstring "\nFile name to write to:")
         FH (open FN "w"))
   (while EN
      (setq EL (entget EN)
            TY (cdr (assoc 0 EL))
            OL (strcat TY "," (cdr (assoc 8 EL))))
      (if (not (null (assoc 62 EL)))
          (setq OL (strcat OL "," (itoa (cdr (assoc 62 EL))))))
      (if (not (null (assoc 6 EL)))
          (setq OL (strcat OL "," (cdr (assoc 6 EL)))))
      (cond
        ((= TY "LINE")
          (write-line OL FH)
          (setq P1 (cdr (assoc 10 EL))
                P2 (cdr (assoc 11 EL))
                OL (strcat (rtos (car P1)) "," (rtos (cadr P1))))
          (write-line OL FH)
          (setq OL (strcat (rtos (car P2)) "," (rtos (cadr P2))))
          (write-line OL FH))
        ((= TY "ARC")
          (write-line OL FH)
          (setq P1 (cdr (assoc 10 EL))
                RD (cdr (assoc 40 EL))
                BA (cdr (assoc 50 EL))
                EA (cdr (assoc 51 EL))
                OL (strcat (rtos (car P1)) "," (rtos (cadr P1)) ","
                     (rtos RD) "," (rtos (r2d BA)) "," (rtos (r2d EA))))
          (write-line OL FH))
        ((= TY "CIRCLE")
          (write-line OL FH)
          (setq P1 (cdr (assoc 10 EL))
                RD (cdr (assoc 40 EL))
                OL (strcat (rtos (car P1)) "," (rtos (cadr P1)) ","
                     (rtos RD)))
          (write-line OL FH))
        ((= TY "TEXT")
          (write-line OL FH)
          (setq P1 (cdr (assoc 10 EL))
                TH (cdr (assoc 40 EL))
                TR (cdr (assoc 50 EL))
                OL (strcat (rtos (car P1)) "," (rtos (cadr P1)) ","
                     (rtos TH) "," (rtos TR)))
          (write-line OL FH)
          (write-line (cdr (assoc 1 EL)) FH))
        ((= TY "INSERT")
          (if (null (assoc 41 EL)) (setq XS 1.0)
                 (setq XS (cdr (assoc 41 EL))))
          (if (null (assoc 42 EL)) (setq YS 1.0)
                 (setq YS (cdr (assoc 42 EL))))
          (if (null (assoc 43 EL)) (setq ZS 1.0)
                 (setq ZS (cdr (assoc 43 EL))))
          (if (and (= XS 1.0) (= YS 1.0) (= ZS 1.0))
                (command "EXPLODE" EN)
                (prompt "\nBlock must have no X,Y,Z scale!"))
        )
        ((or 
             (= TY "POLYLINE")
             (= TY "DIMENSION"))
              (command "EXPLODE" EN))
        (t
          (prompt (strcat "\nUnsupported entity: " TY)))
      )
     (setq EN (entnext EN))
   )
   (setq FH (close FH))
   (prompt "\nGROUT Completes.")
   (princ)
)
