(defun SETUP ()

    ;Get information to set up the graphics screen
    (setq maxspan (getreal "\nDistance between Structures (Feet): "))
    (setq maxvertical (getreal "\nHeight of Structures (Feet): "))
    (setq hscale (getreal "\nEnter Horizontal Scale (100 or 200 feet?):"))
    ;compute vertical scaling factor
    (setq vscale (/ hscale 20.0))

    ;compute maximum and minimum coordinate of the screen to be displayed
    (setq maxh (/ maxspan 2.0))             ;maximum horizontal coordinate
    (setq minh (- 0.0 maxh))                ;minimum horizontal coordinate
    (setq maxv (* vscale maxvertical))      ;maximum vertical coordinate

    ;find the portion below the horizontal axis (always equal one-tenth
    ;of the vertical maximum coordinate
    (setq minv (- 0.0 (* vscale (/ maxvertical 20.0))))
    (setvar "CMDECHO" 0)                    ;turn off echo command
    (setvar "LIMMIN" (list minh minv))      ;change the lower left limit
    (setvar "LIMMAX" (list maxh maxv))      ;change the upper right limit
    (command "UNITS" 2 2 2 2 0 "N")         ;change the unit display
    (command "ZOOM" "A")                    ;display drawing screen

    ;draw horizontal and two vertical axis
    (command "COLOR" "WHITE")
    (command "LINE" (list minh 0.0) (list maxh 0.0) "" )
    (command "LINE" (list maxh 0.0) (list maxh maxv) "" )
    (command "LINE" (list 0.0 0.0) (list 0.0 maxv) "" )

    ;draw tick marks on horizontal axis
    (command "COLOR" "GREEN")
    (setq tick (/ (abs minv) 12.0))          ;compute tick mark height
    (setq step (+ minh hscale))              ;set increment
    (while (< step maxh)
        (command "LINE" (list step 0.0) (list step tick))
        (command "")
        (setq step (+ step hscale))
    )
    ;compute horizontal tick marks on vertical axis
    (command "COLOR" "GREEN")
    (setq tick (- maxh (/ maxh  100.0)))
    (setq vstep (* 20.0 vscale))
    (setq step vstep)
    (while (< step maxv)
        (command "LINE" (list maxh step) (list tick step))
        (command "")
        (setq step (+ step vstep))
    )
    ;draw the left vertical axis
    (command "MIRROR" "C" (list maxh 0.0) (list (- maxh 5.0) maxv)
        "" (list 0.0 0.0) (list 0.0 5.0) "" )
    (command "ZOOM" "E")
    ;(setq SETUP nil)                        ;recover HEAP & STACK memory
)
(defun COMPUTESAG (newspan rulingspan sag )
    (/ (/ (* newspan newspan sag) rulingspan ) rulingspan )
)

(defun INPUT ()
    ;get information to compute sag
    (setq rulingspan (getreal "\nRuling Span (Feet): "))
    (setq clearance (getreal "\nGround Clearance (Feet): "))
    (setq sag (getreal "\nSag (Feet) :"))

    ;select color for the curve to be drawn
    (setq desiredcolor (getstring "\nSelect Color For The Curve : "))
    (command "COLOR" desiredcolor)

    ;compute horizontal step size for computing sag
    (setq stepsize (/ maxh 100.0))

    ;set initial newspan = stepsize
    (setq newspan (* 2.0 stepsize))

    ;draw curve in the right half plane
    (setq oldx 0.0)
    (setq oldy (* clearance vscale))
    (while (<= newspan maxspan)
        (setq newsag (* vscale (computesag newspan rulingspan sag)))
        (setq newx (+ oldx stepsize))
        (setq newy (+ newsag (* clearance vscale)))
        (command "LINE" (list oldx oldy) (list newx newy) )
        (setq oldx newx)
        (setq oldy newy )
        (setq newspan (+ newspan (* 2.0 stepsize)))
    )

    ;compute horizontal step size for computing sag
    (setq stepsize (/ maxh 100.0))
    ;set initial newspan = stepsize
    (setq newspan (* 2.0 stepsize))
    ;draw curve in the left half plane
    (setq oldx 0.0)
    (setq oldy (* clearance vscale))
    (while (<= newspan maxspan)
        (setq newsag (* vscale (computesag newspan rulingspan sag)))
        (setq newx (- oldx stepsize))
        (setq newy (+ newsag (* clearance vscale)))
        (command "LINE" (list oldx oldy) (list newx newy) )
        (setq oldx newx)
        (setq oldy newy )
        (setq newspan (+ newspan (* 2.0 stepsize)))
    )
        (command "REDRAW")
)
(defun LABEL ()
    (setq tsize (/ (abs minv) 4.0))          ; compute text size
    (setq ydec (* 3.0 tsize))
    ;enter & draw titles in the midle of the screen
    ;convert rulingspan from real to string
    (setq dummy (itoa (fix rulingspan)))
    (setq line1 (strcat "RS = " dummy))
    (setq line2 (strcat "MDT "
        (getstring "\nMaximum Design Tension (lbs): ") " lbs" ))
    (setq line3 (strcat "CT " (getstring "\nConductor Type :")))
    (setq line4 (strcat "H-1''=" (itoa (fix hscale)) "'" ))
    (setq line5 "V-1''=20'")

    ;draw titles (5 lines)
    (command "COLOR" "YELLOW")              ;select color for the curve
    ;select ITALICC as the new text style to draw titles
    (command "STYLE" "TEXT1" "ITALICC" "" "" "" "" "" "")
    (setq xpos (/ maxh 70.0))
    (command "TEXT" (list xpos maxv) (* 1.5 tsize ) "" line1)
    (command "TEXT" (list xpos (- maxv ydec)) (* 1.5 tsize) "" line2)
    (command "TEXT" (list xpos (- maxv (* 2.0 ydec)))
        (* 1.5 tsize) "" line3)
    (command "TEXT" (list xpos (- maxv (* 3.0 ydec)))
        (* 1.5 tsize) "" line4)
    (command "TEXT" (list xpos (- maxv (* 4.0 ydec)))
        (* 1.5 tsize) "" line5)

    ;draw label on the horizontal axis
    (command "COLOR" "MAGENTA" )
    (command "STYLE" "TEXT2" "TXT" "" "" "" "" "" "")
    (setq ypos (/ minv 1.5))
    (setq toprint 0)
    (setq xpos minh)
    (while (< xpos maxh)
        (command "TEXT" "C" (list xpos ypos) tsize "" (itoa toprint))
        (setq xpos (+ xpos hscale))
        (setq toprint (+ toprint (fix hscale)))
     )
     (command "TEXT" "C" (list maxh ypos) tsize "" (itoa (fix maxspan)))

    ;draw label on the vertical axis
    (setq xpos (+ maxh (/ maxh 70.0)))
    (setq ypos 0.0)
    (while (< ypos maxvertical)
        (command "TEXT" (list xpos (* ypos vscale)) tsize ""
            (itoa (fix ypos)))
        (setq ypos (+ ypos 20.0))
    )
    (command "TEXT" (list xpos (* maxvertical vscale)) tsize ""
            (itoa (fix maxvertical)))
    (command "ZOOM" "E")
    ;select ROMANC style in GREEN to label individual curve
    (command "STYLE" "TEXT3" "ROMANC" "" "" "" "" "" "")
    (command "COLOR" "GREEN")
)
